@@ -1,17 +1,18 @@
-use super::tag::Tag;
+use std::collections::HashMap;
+use std::ops::Deref;
+
+use rustc_hash::FxHashMap;
+
 use crate::format_element::tag::{Condition, DedentMode};
 use crate::prelude::tag::GroupMode;
 use crate::prelude::*;
-use crate::printer::LineEnding;
 use crate::source_code::SourceCode;
-use crate::{format, write};
 use crate::{
     BufferExtensions, Format, FormatContext, FormatElement, FormatOptions, FormatResult, Formatter,
-    IndentStyle, LineWidth, PrinterOptions,
+    IndentStyle, IndentWidth, LineWidth, PrinterOptions, format, write,
 };
-use rustc_hash::FxHashMap;
-use std::collections::HashMap;
-use std::ops::Deref;
+
+use super::tag::Tag;
 
 /// A formatted document.
 #[derive(Debug, Clone, Eq, PartialEq, Default)]
@@ -21,9 +22,9 @@ pub struct Document {
 
 impl Document {
     /// Sets [`expand`](tag::Group::expand) to [`GroupMode::Propagated`] if the group contains any of:
-    /// - a group with [`expand`](tag::Group::expand) set to [GroupMode::Propagated] or [GroupMode::Expand].
-    /// - a non-soft [line break](FormatElement::Line) with mode [LineMode::Hard], [LineMode::Empty], or [LineMode::Literal].
-    /// - a [FormatElement::ExpandParent]
+    /// - a group with [`expand`](tag::Group::expand) set to [`GroupMode::Propagated`] or [`GroupMode::Expand`].
+    /// - a non-soft [line break](FormatElement::Line) with mode [`LineMode::Hard`], [`LineMode::Empty`], or [`LineMode::Literal`].
+    /// - a [`FormatElement::ExpandParent`]
     ///
     /// [`BestFitting`] elements act as expand boundaries, meaning that the fact that a
     /// [`BestFitting`]'s content expands is not propagated past the [`BestFitting`] element.
@@ -34,15 +35,21 @@ impl Document {
         enum Enclosing<'a> {
             Group(&'a tag::Group),
             ConditionalGroup(&'a tag::ConditionalGroup),
-            FitsExpanded(&'a tag::FitsExpanded),
+            FitsExpanded {
+                tag: &'a tag::FitsExpanded,
+                expands_before: bool,
+            },
             BestFitting,
+            BestFitParenthesize {
+                expanded: bool,
+            },
         }
 
         fn expand_parent(enclosing: &[Enclosing]) {
             match enclosing.last() {
                 Some(Enclosing::Group(group)) => group.propagate_expand(),
                 Some(Enclosing::ConditionalGroup(group)) => group.propagate_expand(),
-                Some(Enclosing::FitsExpanded(fits_expanded)) => fits_expanded.propagate_expand(),
+                Some(Enclosing::FitsExpanded { tag, .. }) => tag.propagate_expand(),
                 _ => {}
             }
         }
@@ -63,6 +70,18 @@ impl Document {
                         Some(Enclosing::Group(group)) => !group.mode().is_flat(),
                         _ => false,
                     },
+                    FormatElement::Tag(Tag::StartBestFitParenthesize { .. }) => {
+                        enclosing.push(Enclosing::BestFitParenthesize { expanded: expands });
+                        expands = false;
+                        continue;
+                    }
+
+                    FormatElement::Tag(Tag::EndBestFitParenthesize) => {
+                        if let Some(Enclosing::BestFitParenthesize { expanded }) = enclosing.pop() {
+                            expands = expanded;
+                        }
+                        continue;
+                    }
                     FormatElement::Tag(Tag::StartConditionalGroup(group)) => {
                         enclosing.push(Enclosing::ConditionalGroup(group));
                         false
@@ -71,42 +90,44 @@ impl Document {
                         Some(Enclosing::ConditionalGroup(group)) => !group.mode().is_flat(),
                         _ => false,
                     },
-                    FormatElement::Interned(interned) => match checked_interned.get(interned) {
-                        Some(interned_expands) => *interned_expands,
-                        None => {
+                    FormatElement::Interned(interned) => {
+                        if let Some(interned_expands) = checked_interned.get(interned) {
+                            *interned_expands
+                        } else {
                             let interned_expands =
                                 propagate_expands(interned, enclosing, checked_interned);
                             checked_interned.insert(interned, interned_expands);
                             interned_expands
                         }
-                    },
-                    FormatElement::BestFitting { variants } => {
+                    }
+                    FormatElement::BestFitting { variants, mode: _ } => {
                         enclosing.push(Enclosing::BestFitting);
 
-                        for variant in variants {
-                            propagate_expands(variant, enclosing, checked_interned);
-                        }
-
-                        // Best fitting acts as a boundary
-                        expands = false;
+                        propagate_expands(variants, enclosing, checked_interned);
                         enclosing.pop();
                         continue;
                     }
                     FormatElement::Tag(Tag::StartFitsExpanded(fits_expanded)) => {
-                        enclosing.push(Enclosing::FitsExpanded(fits_expanded));
+                        enclosing.push(Enclosing::FitsExpanded {
+                            tag: fits_expanded,
+                            expands_before: expands,
+                        });
                         false
                     }
                     FormatElement::Tag(Tag::EndFitsExpanded) => {
-                        enclosing.pop();
-                        // Fits expanded acts as a boundary
-                        expands = false;
+                        if let Some(Enclosing::FitsExpanded { expands_before, .. }) =
+                            enclosing.pop()
+                        {
+                            expands = expands_before;
+                        }
+
                         continue;
                     }
-                    FormatElement::StaticText { text } => text.contains('\n'),
-                    FormatElement::DynamicText { text, .. } => text.contains('\n'),
-                    FormatElement::SourceCodeSlice {
-                        contains_newlines, ..
-                    } => *contains_newlines,
+                    FormatElement::Text {
+                        text: _,
+                        text_width,
+                    } => text_width.is_multiline(),
+                    FormatElement::SourceCodeSlice { text_width, .. } => text_width.is_multiline(),
                     FormatElement::ExpandParent
                     | FormatElement::Line(LineMode::Hard | LineMode::Empty) => true,
                     _ => false,
@@ -114,19 +135,23 @@ impl Document {
 
                 if element_expands {
                     expands = true;
-                    expand_parent(enclosing)
+                    expand_parent(enclosing);
                 }
             }
 
             expands
         }
 
-        let mut enclosing: Vec<Enclosing> = Vec::new();
-        let mut interned: FxHashMap<&Interned, bool> = FxHashMap::default();
+        let mut enclosing = Vec::with_capacity(if self.is_empty() {
+            0
+        } else {
+            self.len().ilog2() as usize
+        });
+        let mut interned = FxHashMap::default();
         propagate_expands(self, &mut enclosing, &mut interned);
     }
 
-    pub fn display<'a>(&'a self, source_code: SourceCode<'a>) -> DisplayDocument {
+    pub fn display<'a>(&'a self, source_code: SourceCode<'a>) -> DisplayDocument<'a> {
         DisplayDocument {
             elements: self.elements.as_slice(),
             source_code,
@@ -197,7 +222,7 @@ impl FormatContext for IrFormatContext<'_> {
         &IrFormatOptions
     }
 
-    fn source_code(&self) -> SourceCode {
+    fn source_code(&self) -> SourceCode<'_> {
         self.source_code
     }
 }
@@ -207,25 +232,29 @@ struct IrFormatOptions;
 
 impl FormatOptions for IrFormatOptions {
     fn indent_style(&self) -> IndentStyle {
-        IndentStyle::Space(2)
+        IndentStyle::Space
+    }
+
+    fn indent_width(&self) -> IndentWidth {
+        IndentWidth::default()
     }
 
     fn line_width(&self) -> LineWidth {
-        LineWidth(80)
+        LineWidth::try_from(80).unwrap()
     }
 
     fn as_print_options(&self) -> PrinterOptions {
         PrinterOptions {
-            tab_width: 2,
-            print_width: self.line_width().into(),
-            line_ending: LineEnding::LineFeed,
-            indent_style: IndentStyle::Space(2),
+            line_width: self.line_width(),
+            indent_style: IndentStyle::Space,
+            ..PrinterOptions::default()
         }
     }
 }
 
 impl Format<IrFormatContext<'_>> for &[FormatElement] {
     fn fmt(&self, f: &mut Formatter<IrFormatContext>) -> FormatResult<()> {
+        #[allow(clippy::enum_glob_use)]
         use Tag::*;
 
         write!(f, [ContentArrayStart])?;
@@ -239,50 +268,53 @@ impl Format<IrFormatContext<'_>> for &[FormatElement] {
         while let Some(element) = iter.next() {
             if !first_element && !in_text && !element.is_end_tag() {
                 // Write a separator between every two elements
-                write!(f, [text(","), soft_line_break_or_space()])?;
+                write!(f, [token(","), soft_line_break_or_space()])?;
             }
 
             first_element = false;
 
             match element {
-                element @ FormatElement::Space
-                | element @ FormatElement::StaticText { .. }
-                | element @ FormatElement::DynamicText { .. }
-                | element @ FormatElement::SourceCodeSlice { .. } => {
-                    if !in_text {
-                        write!(f, [text("\"")])?;
-                    }
-
-                    in_text = true;
-
-                    fn write_escaped(
-                        element: &FormatElement,
-                        f: &mut Formatter<IrFormatContext>,
-                    ) -> FormatResult<()> {
-                        let text = match element {
-                            FormatElement::StaticText { text } => text,
-                            FormatElement::DynamicText { text } => text.as_ref(),
-                            FormatElement::SourceCodeSlice { slice, .. } => {
-                                slice.text(f.context().source_code())
+                element @ (FormatElement::Space
+                | FormatElement::Token { .. }
+                | FormatElement::Text { .. }
+                | FormatElement::SourceCodeSlice { .. }) => {
+                    fn write_escaped(element: &FormatElement, f: &mut Formatter<IrFormatContext>) {
+                        let (text, text_width) = match element {
+                            #[expect(clippy::cast_possible_truncation)]
+                            FormatElement::Token { text } => {
+                                (*text, TextWidth::Width(Width::new(text.len() as u32)))
+                            }
+                            FormatElement::Text { text, text_width } => {
+                                (text.as_ref(), *text_width)
+                            }
+                            FormatElement::SourceCodeSlice { slice, text_width } => {
+                                (slice.text(f.context().source_code()), *text_width)
                             }
                             _ => unreachable!(),
                         };
 
                         if text.contains('"') {
-                            f.write_element(FormatElement::DynamicText {
+                            f.write_element(FormatElement::Text {
                                 text: text.replace('"', r#"\""#).into(),
-                            })
+                                text_width,
+                            });
                         } else {
-                            f.write_element(element.clone())
+                            f.write_element(element.clone());
                         }
                     }
 
+                    if !in_text {
+                        write!(f, [token("\"")])?;
+                    }
+
+                    in_text = true;
+
                     match element {
                         FormatElement::Space => {
-                            write!(f, [text(" ")])?;
+                            write!(f, [token(" ")])?;
                         }
                         element if element.is_text() => {
-                            write_escaped(element, f)?;
+                            write_escaped(element, f);
                         }
                         _ => unreachable!(),
                     }
@@ -290,60 +322,60 @@ impl Format<IrFormatContext<'_>> for &[FormatElement] {
                     let is_next_text = iter.peek().is_some_and(|e| e.is_text() || e.is_space());
 
                     if !is_next_text {
-                        write!(f, [text("\"")])?;
+                        write!(f, [token("\"")])?;
                         in_text = false;
                     }
                 }
 
                 FormatElement::Line(mode) => match mode {
                     LineMode::SoftOrSpace => {
-                        write!(f, [text("soft_line_break_or_space")])?;
+                        write!(f, [token("soft_line_break_or_space")])?;
                     }
                     LineMode::Soft => {
-                        write!(f, [text("soft_line_break")])?;
+                        write!(f, [token("soft_line_break")])?;
                     }
                     LineMode::Hard => {
-                        write!(f, [text("hard_line_break")])?;
+                        write!(f, [token("hard_line_break")])?;
                     }
                     LineMode::Empty => {
-                        write!(f, [text("empty_line")])?;
+                        write!(f, [token("empty_line")])?;
                     }
                 },
                 FormatElement::ExpandParent => {
-                    write!(f, [text("expand_parent")])?;
+                    write!(f, [token("expand_parent")])?;
                 }
 
                 FormatElement::SourcePosition(position) => {
-                    write!(
-                        f,
-                        [dynamic_text(
-                            &std::format!("source_position({:?})", position),
-                            None
-                        )]
-                    )?;
+                    write!(f, [text(&std::format!("source_position({position:?})"))])?;
                 }
 
                 FormatElement::LineSuffixBoundary => {
-                    write!(f, [text("line_suffix_boundary")])?;
+                    write!(f, [token("line_suffix_boundary")])?;
                 }
 
-                FormatElement::BestFitting { variants } => {
-                    write!(f, [text("best_fitting([")])?;
+                FormatElement::BestFitting { variants, mode } => {
+                    write!(f, [token("best_fitting(")])?;
+
+                    if *mode != BestFittingMode::FirstLine {
+                        write!(f, [text(&std::format!("mode: {mode:?}, "))])?;
+                    }
+
+                    write!(f, [token("[")])?;
                     f.write_elements([
                         FormatElement::Tag(StartIndent),
                         FormatElement::Line(LineMode::Hard),
-                    ])?;
+                    ]);
 
                     for variant in variants {
-                        write!(f, [variant.deref(), hard_line_break()])?;
+                        write!(f, [variant, hard_line_break()])?;
                     }
 
                     f.write_elements([
                         FormatElement::Tag(EndIndent),
                         FormatElement::Line(LineMode::Hard),
-                    ])?;
+                    ]);
 
-                    write!(f, [text("])")])?;
+                    write!(f, [token("])")])?;
                 }
 
                 FormatElement::Interned(interned) => {
@@ -357,20 +389,14 @@ impl Format<IrFormatContext<'_>> for &[FormatElement] {
                             write!(
                                 f,
                                 [
-                                    dynamic_text(&std::format!("<interned {index}>"), None),
+                                    text(&std::format!("<interned {index}>")),
                                     space(),
-                                    &interned.deref(),
+                                    &&**interned,
                                 ]
                             )?;
                         }
                         Some(reference) => {
-                            write!(
-                                f,
-                                [dynamic_text(
-                                    &std::format!("<ref interned *{reference}>"),
-                                    None
-                                )]
-                            )?;
+                            write!(f, [text(&std::format!("<ref interned *{reference}>"))])?;
                         }
                     }
                 }
@@ -388,9 +414,9 @@ impl Format<IrFormatContext<'_>> for &[FormatElement] {
                                 write!(
                                     f,
                                     [
-                                        text("<END_TAG_WITHOUT_START<"),
-                                        dynamic_text(&std::format!("{:?}", tag.kind()), None),
-                                        text(">>"),
+                                        token("<END_TAG_WITHOUT_START<"),
+                                        text(&std::format!("{:?}", tag.kind())),
+                                        token(">>"),
                                     ]
                                 )?;
                                 first_element = false;
@@ -401,13 +427,13 @@ impl Format<IrFormatContext<'_>> for &[FormatElement] {
                                     f,
                                     [
                                         ContentArrayEnd,
-                                        text(")"),
+                                        token(")"),
                                         soft_line_break_or_space(),
-                                        text("ERROR<START_END_TAG_MISMATCH<start: "),
-                                        dynamic_text(&std::format!("{start_kind:?}"), None),
-                                        text(", end: "),
-                                        dynamic_text(&std::format!("{:?}", tag.kind()), None),
-                                        text(">>")
+                                        token("ERROR<START_END_TAG_MISMATCH<start: "),
+                                        text(&std::format!("{start_kind:?}")),
+                                        token(", end: "),
+                                        text(&std::format!("{:?}", tag.kind())),
+                                        token(">>")
                                     ]
                                 )?;
                                 first_element = false;
@@ -421,7 +447,7 @@ impl Format<IrFormatContext<'_>> for &[FormatElement] {
 
                     match tag {
                         StartIndent => {
-                            write!(f, [text("indent(")])?;
+                            write!(f, [token("indent(")])?;
                         }
 
                         StartDedent(mode) => {
@@ -430,62 +456,77 @@ impl Format<IrFormatContext<'_>> for &[FormatElement] {
                                 DedentMode::Root => "dedentRoot",
                             };
 
-                            write!(f, [text(label), text("(")])?;
+                            write!(f, [token(label), token("(")])?;
                         }
 
                         StartAlign(tag::Align(count)) => {
                             write!(
                                 f,
                                 [
-                                    text("align("),
-                                    dynamic_text(&count.to_string(), None),
-                                    text(","),
+                                    token("align("),
+                                    text(&count.to_string()),
+                                    token(","),
                                     space(),
                                 ]
                             )?;
                         }
 
-                        StartLineSuffix => {
-                            write!(f, [text("line_suffix(")])?;
+                        StartLineSuffix { reserved_width } => {
+                            write!(
+                                f,
+                                [
+                                    token("line_suffix("),
+                                    text(&std::format!("{reserved_width:?}")),
+                                    token(","),
+                                    space(),
+                                ]
+                            )?;
                         }
 
                         StartVerbatim(_) => {
-                            write!(f, [text("verbatim(")])?;
+                            write!(f, [token("verbatim(")])?;
                         }
 
                         StartGroup(group) => {
-                            write!(f, [text("group(")])?;
+                            write!(f, [token("group(")])?;
 
                             if let Some(group_id) = group.id() {
                                 write!(
                                     f,
-                                    [
-                                        dynamic_text(&std::format!("\"{group_id:?}\""), None),
-                                        text(","),
-                                        space(),
-                                    ]
+                                    [text(&std::format!("\"{group_id:?}\"")), token(","), space(),]
                                 )?;
                             }
 
                             match group.mode() {
                                 GroupMode::Flat => {}
                                 GroupMode::Expand => {
-                                    write!(f, [text("expand: true,"), space()])?;
+                                    write!(f, [token("expand: true,"), space()])?;
                                 }
                                 GroupMode::Propagated => {
-                                    write!(f, [text("expand: propagated,"), space()])?;
+                                    write!(f, [token("expand: propagated,"), space()])?;
                                 }
                             }
                         }
 
+                        StartBestFitParenthesize { id } => {
+                            write!(f, [token("best_fit_parenthesize(")])?;
+
+                            if let Some(group_id) = id {
+                                write!(
+                                    f,
+                                    [text(&std::format!("\"{group_id:?}\"")), token(","), space(),]
+                                )?;
+                            }
+                        }
+
                         StartConditionalGroup(group) => {
                             write!(
                                 f,
                                 [
-                                    text("conditional_group(condition:"),
+                                    token("conditional_group(condition:"),
                                     space(),
                                     group.condition(),
-                                    text(","),
+                                    token(","),
                                     space()
                                 ]
                             )?;
@@ -493,10 +534,10 @@ impl Format<IrFormatContext<'_>> for &[FormatElement] {
                             match group.mode() {
                                 GroupMode::Flat => {}
                                 GroupMode::Expand => {
-                                    write!(f, [text("expand: true,"), space()])?;
+                                    write!(f, [token("expand: true,"), space()])?;
                                 }
                                 GroupMode::Propagated => {
-                                    write!(f, [text("expand: propagated,"), space()])?;
+                                    write!(f, [token("expand: propagated,"), space()])?;
                                 }
                             }
                         }
@@ -505,9 +546,9 @@ impl Format<IrFormatContext<'_>> for &[FormatElement] {
                             write!(
                                 f,
                                 [
-                                    text("indent_if_group_breaks("),
-                                    dynamic_text(&std::format!("\"{id:?}\""), None),
-                                    text(","),
+                                    token("indent_if_group_breaks("),
+                                    text(&std::format!("\"{id:?}\"")),
+                                    token(","),
                                     space(),
                                 ]
                             )?;
@@ -516,21 +557,17 @@ impl Format<IrFormatContext<'_>> for &[FormatElement] {
                         StartConditionalContent(condition) => {
                             match condition.mode {
                                 PrintMode::Flat => {
-                                    write!(f, [text("if_group_fits_on_line(")])?;
+                                    write!(f, [token("if_group_fits_on_line(")])?;
                                 }
                                 PrintMode::Expanded => {
-                                    write!(f, [text("if_group_breaks(")])?;
+                                    write!(f, [token("if_group_breaks(")])?;
                                 }
                             }
 
                             if let Some(group_id) = condition.group_id {
                                 write!(
                                     f,
-                                    [
-                                        dynamic_text(&std::format!("\"{group_id:?}\""), None),
-                                        text(","),
-                                        space(),
-                                    ]
+                                    [text(&std::format!("\"{group_id:?}\"")), token(","), space()]
                                 )?;
                             }
                         }
@@ -539,44 +576,44 @@ impl Format<IrFormatContext<'_>> for &[FormatElement] {
                             write!(
                                 f,
                                 [
-                                    text("label("),
-                                    dynamic_text(&std::format!("\"{label_id:?}\""), None),
-                                    text(","),
+                                    token("label("),
+                                    text(&std::format!("\"{label_id:?}\"")),
+                                    token(","),
                                     space(),
                                 ]
                             )?;
                         }
 
                         StartFill => {
-                            write!(f, [text("fill(")])?;
+                            write!(f, [token("fill(")])?;
                         }
 
                         StartFitsExpanded(tag::FitsExpanded {
                             condition,
                             propagate_expand,
                         }) => {
-                            write!(f, [text("fits_expanded(propagate_expand:"), space()])?;
+                            write!(f, [token("fits_expanded(propagate_expand:"), space()])?;
 
                             if propagate_expand.get() {
-                                write!(f, [text("true")])?;
+                                write!(f, [token("true")])?;
                             } else {
-                                write!(f, [text("false")])?;
+                                write!(f, [token("false")])?;
                             }
 
-                            write!(f, [text(","), space()])?;
+                            write!(f, [token(","), space()])?;
 
                             if let Some(condition) = condition {
                                 write!(
                                     f,
-                                    [text("condition:"), space(), condition, text(","), space()]
+                                    [token("condition:"), space(), condition, token(","), space()]
                                 )?;
                             }
                         }
 
-                        StartEntry => {
+                        StartEntry | StartBestFittingEntry => {
                             // handled after the match for all start tags
                         }
-                        EndEntry => write!(f, [ContentArrayEnd])?,
+                        EndEntry | EndBestFittingEntry => write!(f, [ContentArrayEnd])?,
 
                         EndFill
                         | EndLabelled
@@ -586,13 +623,14 @@ impl Format<IrFormatContext<'_>> for &[FormatElement] {
                         | EndIndent
                         | EndGroup
                         | EndConditionalGroup
+                        | EndBestFitParenthesize
                         | EndLineSuffix
                         | EndDedent
                         | EndFitsExpanded
                         | EndVerbatim => {
-                            write!(f, [ContentArrayEnd, text(")")])?;
+                            write!(f, [ContentArrayEnd, token(")")])?;
                         }
-                    };
+                    }
 
                     if tag.is_start() {
                         write!(f, [ContentArrayStart])?;
@@ -606,9 +644,9 @@ impl Format<IrFormatContext<'_>> for &[FormatElement] {
                 f,
                 [
                     ContentArrayEnd,
-                    text(")"),
+                    token(")"),
                     soft_line_break_or_space(),
-                    dynamic_text(&std::format!("<START_WITHOUT_END<{top:?}>>"), None),
+                    text(&std::format!("<START_WITHOUT_END<{top:?}>>")),
                 ]
             )?;
         }
@@ -621,15 +659,17 @@ struct ContentArrayStart;
 
 impl Format<IrFormatContext<'_>> for ContentArrayStart {
     fn fmt(&self, f: &mut Formatter<IrFormatContext>) -> FormatResult<()> {
-        use Tag::*;
+        use Tag::{StartGroup, StartIndent};
 
-        write!(f, [text("[")])?;
+        write!(f, [token("[")])?;
 
         f.write_elements([
             FormatElement::Tag(StartGroup(tag::Group::new())),
             FormatElement::Tag(StartIndent),
             FormatElement::Line(LineMode::Soft),
-        ])
+        ]);
+
+        Ok(())
     }
 }
 
@@ -637,31 +677,32 @@ struct ContentArrayEnd;
 
 impl Format<IrFormatContext<'_>> for ContentArrayEnd {
     fn fmt(&self, f: &mut Formatter<IrFormatContext>) -> FormatResult<()> {
-        use Tag::*;
+        use Tag::{EndGroup, EndIndent};
         f.write_elements([
             FormatElement::Tag(EndIndent),
             FormatElement::Line(LineMode::Soft),
             FormatElement::Tag(EndGroup),
-        ])?;
+        ]);
 
-        write!(f, [text("]")])
+        write!(f, [token("]")])
     }
 }
 
 impl FormatElements for [FormatElement] {
     fn will_break(&self) -> bool {
-        use Tag::*;
         let mut ignore_depth = 0usize;
 
         for element in self {
             match element {
                 // Line suffix
                 // Ignore if any of its content breaks
-                FormatElement::Tag(StartLineSuffix) => {
+                FormatElement::Tag(
+                    Tag::StartLineSuffix { reserved_width: _ } | Tag::StartFitsExpanded(_),
+                ) => {
                     ignore_depth += 1;
                 }
-                FormatElement::Tag(EndLineSuffix) => {
-                    ignore_depth -= 1;
+                FormatElement::Tag(Tag::EndLineSuffix | Tag::EndFitsExpanded) => {
+                    ignore_depth = ignore_depth.saturating_sub(1);
                 }
                 FormatElement::Interned(interned) if ignore_depth == 0 => {
                     if interned.will_break() {
@@ -687,9 +728,6 @@ impl FormatElements for [FormatElement] {
     }
 
     fn start_tag(&self, kind: TagKind) -> Option<&Tag> {
-        // Assert that the document ends at a tag with the specified kind;
-        let _ = self.end_tag(kind)?;
-
         fn traverse_slice<'a>(
             slice: &'a [FormatElement],
             kind: TagKind,
@@ -704,9 +742,8 @@ impl FormatElements for [FormatElement] {
                                 return None;
                             } else if *depth == 1 {
                                 return Some(tag);
-                            } else {
-                                *depth -= 1;
                             }
+                            *depth -= 1;
                         } else {
                             *depth += 1;
                         }
@@ -731,6 +768,8 @@ impl FormatElements for [FormatElement] {
 
             None
         }
+        // Assert that the document ends at a tag with the specified kind;
+        let _ = self.end_tag(kind)?;
 
         let mut depth = 0usize;
 
@@ -745,22 +784,22 @@ impl FormatElements for [FormatElement] {
 impl Format<IrFormatContext<'_>> for Condition {
     fn fmt(&self, f: &mut Formatter<IrFormatContext>) -> FormatResult<()> {
         match (self.mode, self.group_id) {
-            (PrintMode::Flat, None) => write!(f, [text("if_fits_on_line")]),
+            (PrintMode::Flat, None) => write!(f, [token("if_fits_on_line")]),
             (PrintMode::Flat, Some(id)) => write!(
                 f,
                 [
-                    text("if_group_fits_on_line("),
-                    dynamic_text(&std::format!("\"{id:?}\""), None),
-                    text(")")
+                    token("if_group_fits_on_line("),
+                    text(&std::format!("\"{id:?}\"")),
+                    token(")")
                 ]
             ),
-            (PrintMode::Expanded, None) => write!(f, [text("if_breaks")]),
+            (PrintMode::Expanded, None) => write!(f, [token("if_breaks")]),
             (PrintMode::Expanded, Some(id)) => write!(
                 f,
                 [
-                    text("if_group_breaks("),
-                    dynamic_text(&std::format!("\"{id:?}\""), None),
-                    text(")")
+                    token("if_group_breaks("),
+                    text(&std::format!("\"{id:?}\"")),
+                    token(")")
                 ]
             ),
         }
@@ -769,11 +808,12 @@ impl Format<IrFormatContext<'_>> for Condition {
 
 #[cfg(test)]
 mod tests {
-    use crate::prelude::*;
-    use crate::{format, format_args, write};
-    use crate::{SimpleFormatContext, SourceCode};
     use ruff_text_size::{TextRange, TextSize};
 
+    use crate::prelude::*;
+    use crate::{SimpleFormatContext, SourceCode};
+    use crate::{format, format_args, write};
+
     #[test]
     fn display_elements() {
         let formatted = format!(
@@ -782,11 +822,11 @@ mod tests {
                 write!(
                     f,
                     [group(&format_args![
-                        text("("),
+                        token("("),
                         soft_block_indent(&format_args![
-                            text("Some longer content"),
+                            token("Some longer content"),
                             space(),
-                            text("That should ultimately break"),
+                            token("That should ultimately break"),
                         ])
                     ])]
                 )
@@ -815,7 +855,7 @@ mod tests {
     fn escapes_quotes() {
         let formatted = format!(
             SimpleFormatContext::default(),
-            [text(r#""""Python docstring""""#)]
+            [token(r#""""Python docstring""""#)]
         )
         .unwrap();
 
@@ -836,17 +876,11 @@ mod tests {
                 write!(
                     f,
                     [group(&format_args![
-                        text("("),
+                        token("("),
                         soft_block_indent(&format_args![
-                            source_text_slice(
-                                TextRange::at(TextSize::new(0), TextSize::new(19)),
-                                ContainsNewlines::No
-                            ),
+                            source_text_slice(TextRange::at(TextSize::new(0), TextSize::new(19))),
                             space(),
-                            source_text_slice(
-                                TextRange::at(TextSize::new(20), TextSize::new(28)),
-                                ContainsNewlines::No
-                            ),
+                            source_text_slice(TextRange::at(TextSize::new(20), TextSize::new(28))),
                         ])
                     ])]
                 )
@@ -876,16 +910,16 @@ mod tests {
         use Tag::*;
 
         let document = Document::from(vec![
-            FormatElement::StaticText { text: "[" },
+            FormatElement::Token { text: "[" },
             FormatElement::Tag(StartGroup(tag::Group::new())),
             FormatElement::Tag(StartIndent),
             FormatElement::Line(LineMode::Soft),
-            FormatElement::StaticText { text: "a" },
+            FormatElement::Token { text: "a" },
             // Close group instead of indent
             FormatElement::Tag(EndGroup),
             FormatElement::Line(LineMode::Soft),
             FormatElement::Tag(EndIndent),
-            FormatElement::StaticText { text: "]" },
+            FormatElement::Token { text: "]" },
             // End tag without start
             FormatElement::Tag(EndIndent),
             // Start tag without an end
