@@ -6,7 +6,8 @@ use ruff_db::parsed::parsed_module;
 use ruff_python_ast::visitor::source_order::{self, SourceOrderVisitor, TraversalSignal};
 use ruff_python_ast::{AnyNodeRef, Expr, Stmt};
 use ruff_text_size::{Ranged, TextRange, TextSize};
-use ty_python_semantic::types::{Type, inlay_hint_function_argument_details};
+use ty_python_semantic::types::Type;
+use ty_python_semantic::types::ide_support::inlay_hint_function_argument_details;
 use ty_python_semantic::{HasType, SemanticModel};
 
 #[derive(Debug, Clone)]
@@ -264,6 +265,15 @@ impl SourceOrderVisitor<'_> for InlayHintVisitor<'_, '_> {
                 }
                 source_order::walk_expr(self, expr);
             }
+            Expr::Attribute(attribute) => {
+                if self.in_assignment {
+                    if attribute.ctx.is_store() {
+                        let ty = expr.inferred_type(&self.model);
+                        self.add_type_hint(expr.range().end(), ty);
+                    }
+                }
+                source_order::walk_expr(self, expr);
+            }
             Expr::Call(call) => {
                 let argument_names =
                     inlay_hint_function_argument_details(self.db, &self.model, call)
@@ -292,7 +302,6 @@ mod tests {
 
     use insta::assert_snapshot;
     use ruff_db::{
-        Db as _,
         files::{File, system_path_to_file},
         source::source_text,
     };
@@ -301,9 +310,6 @@ mod tests {
 
     use ruff_db::system::{DbWithWritableSystem, SystemPathBuf};
     use ty_project::ProjectMetadata;
-    use ty_python_semantic::{
-        Program, ProgramSettings, PythonPlatform, PythonVersionWithSource, SearchPathSettings,
-    };
 
     pub(super) fn inlay_hint_test(source: &str) -> InlayHintTest {
         const START: &str = "<START>";
@@ -314,6 +320,8 @@ mod tests {
             SystemPathBuf::from("/"),
         ));
 
+        db.init_program().unwrap();
+
         let source = dedent(source);
 
         let start = source.find(START);
@@ -335,19 +343,6 @@ mod tests {
 
         let file = system_path_to_file(&db, "main.py").expect("newly written file to existing");
 
-        let search_paths = SearchPathSettings::new(vec![SystemPathBuf::from("/")])
-            .to_search_paths(db.system(), db.vendored())
-            .expect("Valid search path settings");
-
-        Program::from_settings(
-            &db,
-            ProgramSettings {
-                python_version: PythonVersionWithSource::default(),
-                python_platform: PythonPlatform::default(),
-                search_paths,
-            },
-        );
-
         InlayHintTest { db, file, range }
     }
 
@@ -436,6 +431,31 @@ mod tests {
         ");
     }
 
+    #[test]
+    fn test_assign_attribute_of_instance() {
+        let test = inlay_hint_test(
+            "
+            class A:
+                def __init__(self, y):
+                    self.x = 1
+                    self.y = y
+
+            a = A(2)
+            a.y = 3
+            ",
+        );
+
+        assert_snapshot!(test.inlay_hints(), @r"
+        class A:
+            def __init__(self, y):
+                self.x[: Literal[1]] = 1
+                self.y[: Unknown] = y
+
+        a[: A] = A([y=]2)
+        a.y[: Literal[3]] = 3
+        ");
+    }
+
     #[test]
     fn test_disabled_variable_types() {
         let test = inlay_hint_test("x = 1");
