@@ -523,6 +523,24 @@ impl Display for DisplayRepresentation<'_> {
             Type::KnownBoundMethod(KnownBoundMethodType::PathOpen) => {
                 f.write_str("bound method `Path.open`")
             }
+            Type::KnownBoundMethod(KnownBoundMethodType::ConstraintSetRange) => {
+                f.write_str("bound method `ConstraintSet.range`")
+            }
+            Type::KnownBoundMethod(KnownBoundMethodType::ConstraintSetAlways) => {
+                f.write_str("bound method `ConstraintSet.always`")
+            }
+            Type::KnownBoundMethod(KnownBoundMethodType::ConstraintSetNever) => {
+                f.write_str("bound method `ConstraintSet.never`")
+            }
+            Type::KnownBoundMethod(KnownBoundMethodType::ConstraintSetImpliesSubtypeOf(_)) => {
+                f.write_str("bound method `ConstraintSet.implies_subtype_of`")
+            }
+            Type::KnownBoundMethod(KnownBoundMethodType::ConstraintSetSatisfies(_)) => {
+                f.write_str("bound method `ConstraintSet.satisfies`")
+            }
+            Type::KnownBoundMethod(KnownBoundMethodType::ConstraintSetSatisfiedByAllTypeVars(
+                _,
+            )) => f.write_str("bound method `ConstraintSet.satisfied_by_all_typevars`"),
             Type::WrapperDescriptor(kind) => {
                 let (method, object) = match kind {
                     WrapperDescriptorKind::FunctionTypeDunderGet => ("__get__", "function"),
