@@ -2,12 +2,15 @@ mod completion;
 mod diagnostic;
 mod doc_highlights;
 mod document_symbols;
+mod execute_command;
 mod goto_declaration;
 mod goto_definition;
 mod goto_references;
 mod goto_type_definition;
 mod hover;
 mod inlay_hints;
+mod prepare_rename;
+mod rename;
 mod selection_range;
 mod semantic_tokens;
 mod semantic_tokens_range;
@@ -20,12 +23,15 @@ pub(super) use completion::CompletionRequestHandler;
 pub(super) use diagnostic::DocumentDiagnosticRequestHandler;
 pub(super) use doc_highlights::DocumentHighlightRequestHandler;
 pub(super) use document_symbols::DocumentSymbolRequestHandler;
+pub(super) use execute_command::ExecuteCommand;
 pub(super) use goto_declaration::GotoDeclarationRequestHandler;
 pub(super) use goto_definition::GotoDefinitionRequestHandler;
 pub(super) use goto_references::ReferencesRequestHandler;
 pub(super) use goto_type_definition::GotoTypeDefinitionRequestHandler;
 pub(super) use hover::HoverRequestHandler;
 pub(super) use inlay_hints::InlayHintRequestHandler;
+pub(super) use prepare_rename::PrepareRenameRequestHandler;
+pub(super) use rename::RenameRequestHandler;
 pub(super) use selection_range::SelectionRangeRequestHandler;
 pub(super) use semantic_tokens::SemanticTokensRequestHandler;
 pub(super) use semantic_tokens_range::SemanticTokensRangeRequestHandler;
@@ -33,3 +39,5 @@ pub(super) use shutdown::ShutdownHandler;
 pub(super) use signature_help::SignatureHelpRequestHandler;
 pub(super) use workspace_diagnostic::WorkspaceDiagnosticRequestHandler;
 pub(super) use workspace_symbols::WorkspaceSymbolRequestHandler;
+
+pub use workspace_diagnostic::{PartialWorkspaceProgress, PartialWorkspaceProgressParams};
