@@ -1,10 +1,15 @@
 #!/usr/bin/env python3
-"""Check two versions of ruff against a corpus of open-source code.
+"""
+**DEPRECATED** This script is being replaced by the ruff-ecosystem package.
+
+
+Check two versions of ruff against a corpus of open-source code.
 
 Example usage:
 
     scripts/check_ecosystem.py <path/to/ruff1> <path/to/ruff2>
 """
+
 from __future__ import annotations
 
 import argparse
@@ -99,16 +104,46 @@ class Repository(NamedTuple):
             stdout=PIPE,
         )
         git_sha_stdout, _ = await git_sha_process.communicate()
-        assert (
-            await git_sha_process.wait() == 0
-        ), f"Failed to retrieve commit sha at {checkout_dir}"
+        assert await git_sha_process.wait() == 0, (
+            f"Failed to retrieve commit sha at {checkout_dir}"
+        )
         return git_sha_stdout.decode().strip()
 
 
+# Repositories to check
+# We check most repositories with the default ruleset instead of all rules to avoid
+# noisy reports when new rules are added; see https://github.com/astral-sh/ruff/pull/3590
 REPOSITORIES: list[Repository] = [
     Repository("DisnakeDev", "disnake", "master"),
+    Repository("PostHog", "HouseWatch", "main"),
+    Repository("RasaHQ", "rasa", "main"),
+    Repository("Snowflake-Labs", "snowcli", "main"),
+    Repository("aiven", "aiven-client", "main"),
+    Repository("alteryx", "featuretools", "main"),
     Repository("apache", "airflow", "main", select="ALL"),
+    Repository("apache", "superset", "master", select="ALL"),
+    Repository("aws", "aws-sam-cli", "develop"),
+    Repository("binary-husky", "gpt_academic", "master"),
+    Repository("bloomberg", "pytest-memray", "main"),
     Repository("bokeh", "bokeh", "branch-3.3", select="ALL"),
+    # Disabled due to use of explicit `select` with `E999`, which has been removed.
+    # See: https://github.com/astral-sh/ruff/pull/12129
+    # Repository("demisto", "content", "master"),
+    Repository("docker", "docker-py", "main"),
+    Repository("facebookresearch", "chameleon", "main"),
+    Repository("freedomofpress", "securedrop", "develop"),
+    Repository("fronzbot", "blinkpy", "dev"),
+    Repository("ibis-project", "ibis", "master"),
+    Repository("ing-bank", "probatus", "main"),
+    Repository("jrnl-org", "jrnl", "develop"),
+    Repository("langchain-ai", "langchain", "main"),
+    Repository("latchbio", "latch", "main"),
+    Repository("lnbits", "lnbits", "main"),
+    Repository("milvus-io", "pymilvus", "master"),
+    Repository("mlflow", "mlflow", "master"),
+    Repository("model-bakers", "model_bakery", "main"),
+    Repository("pandas-dev", "pandas", "main"),
+    Repository("prefecthq", "prefect", "main"),
     Repository("pypa", "build", "main"),
     Repository("pypa", "cibuildwheel", "main"),
     Repository("pypa", "pip", "main"),
@@ -116,9 +151,15 @@ REPOSITORIES: list[Repository] = [
     Repository("python", "mypy", "master"),
     Repository("python", "typeshed", "main", select="PYI"),
     Repository("python-poetry", "poetry", "master"),
+    Repository("qdrant", "qdrant-client", "master"),
+    Repository("reflex-dev", "reflex", "main"),
+    Repository("rotki", "rotki", "develop"),
     Repository("scikit-build", "scikit-build", "main"),
     Repository("scikit-build", "scikit-build-core", "main"),
+    Repository("sphinx-doc", "sphinx", "master"),
+    Repository("spruceid", "siwe-py", "main"),
     Repository("tiangolo", "fastapi", "master"),
+    Repository("yandex", "ch-backup", "main"),
     Repository("zulip", "zulip", "main", select="ALL"),
 ]
 
@@ -149,7 +190,7 @@ async def check(
     if exclude:
         ruff_args.extend(["--exclude", exclude])
     if show_fixes:
-        ruff_args.extend(["--show-fixes", "--ecosystem-ci"])
+        ruff_args.extend(["--show-fixes"])
 
     start = time.time()
     proc = await create_subprocess_exec(
@@ -301,7 +342,6 @@ DIFF_LINE_RE = re.compile(
     r"^(?P<pre>[+-]) (?P<inner>(?P<path>[^:]+):(?P<lnum>\d+):\d+:) (?P<post>.*)$",
 )
 
-
 T = TypeVar("T")
 
 
@@ -409,7 +449,7 @@ async def main(
 
                     if matches is None:
                         # Handle case where there are no regex matches e.g.
-                        # +                 "?application=AIRFLOW&authenticator=TEST_AUTH&role=TEST_ROLE&warehouse=TEST_WAREHOUSE" # noqa: E501, ERA001
+                        # +                 "?application=AIRFLOW&authenticator=TEST_AUTH&role=TEST_ROLE&warehouse=TEST_WAREHOUSE"
                         # Which was found in local testing
                         continue
 
@@ -488,7 +528,8 @@ if __name__ == "__main__":
     else:
         logging.basicConfig(level=logging.INFO)
 
-    loop = asyncio.get_event_loop()
+    loop = asyncio.new_event_loop()
+    asyncio.set_event_loop(loop)
     if args.checkouts:
         args.checkouts.mkdir(exist_ok=True, parents=True)
     main_task = asyncio.ensure_future(
